/*===========================================================================*
 * image_window.c                                                            *
 *                                                                           *
 *      Procedures to display Conoscopic Image                               *
 *                                                                           *
 * EXPORTED PROCEDURES:                                                      *
 *    GtkWidget* create_ImageWindow ()                                       *
 *                                                            (by A. Miyake) *
 *===========================================================================*/

#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "cono_map.h"
#include "widget_set.h"

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include "intl.h"

/*==================*
 * GLOBAL VARIABLES *   
 *==================*/
gint ImageMargin = 5;
gint mpeg_image_size;
gfloat FrameRate;

/*================================================*
 * EXTENAL VARIABLES (defined in image_drawing.c) *
 *================================================*/
extern GdkPixmap *pixmap;
extern gint image_size;
extern gint total_image_size;
extern gfloat gamma0;
extern gfloat bright0;
extern gfloat phi0;
extern gfloat speed0;

/*==================*
 * STATIC VARIABLES *
 *==================*/
static GtkWidget *ImageArea;
static GtkWidget *AngleScale;
static guint AnimationTimeout;
static GTimer* AnimationTimer;
static gfloat AnimationSpeed;
static gfloat AnimationAngle;
static gboolean AnimatingNow = FALSE; 
static gint AnimationInterval;


/*====================*
 * EXTERNAL FUNCTIONS *   
 *====================*/
extern void change_brightness(gfloat); /* from image_drawing.c */
extern void change_gamma(gfloat);      /* from image_drawing.c */
extern void image_memory_free();       /* from image_drawing.c */
extern void set_animation_image_index(gfloat);  /* from image_drawing.c */
extern gchar* get_file_extention(gchar* file);  /* from save_file.c */
extern void save_image(gchar* file, gchar* ext);/* from save_file.c */

extern void  MessageDialog(gchar* );  /*from main_window.c */

/*===============================* 
 * INTERNAL FUNCTIONS(Callbacks) *
 *===============================*/
static gint expose_event (GtkWidget*, GdkEventExpose *);
static void ImageWindow_destroy (GtkObject*, gpointer);
static void Brightness_draw (GtkWidget*, gpointer);
static void Gamma_draw (GtkWidget*, gpointer);
static void SaveButton_clicked (GtkButton*, gpointer);
static void PrintButton_clicked (GtkButton*, gpointer);
static void OKButton_clicked (GtkButton*, gpointer);
static void expose_angle_scale (GtkWidget *, GdkEventExpose *);
static void Angle_draw (GtkWidget *, gpointer);
static void Speed_draw (GtkWidget *, gpointer);
static void StartButton_clicked (GtkButton*, gpointer);
static void StopButton_clicked (GtkButton*, gpointer);

/*================================================* 
 * Main Internal Functions(NonCallback Functions) *
 *================================================*/
static void draw_image();
static gboolean animating(gpointer data);
static void fdialog_save(GtkWidget *widget, gpointer fdialog);
static gboolean mpeg_size_input();
static void create_text_entry_dialog( gchar* title, gchar* label_str, 
	gchar* default_entry_str,  gchar* action_str,  gchar** get_Str);

/*===========================================================================*
 * (EXPORTED FUNCTION)                                                       *
 *                                                                           *
 *    create_ImageWindow                                                     *
 *===========================================================================*/
GtkWidget*
create_ImageWindow ()
{
  GtkWidget *ImageWindow;
  GtkWidget *ImageHBox, *ImageVBox;
  GtkWidget *ImageFrame;
  GtkWidget *ImageControlBox;
  GtkWidget *ImageControlButtonBox;
  GtkWidget *ActionBox;

  GtkWidget *SaveButton;
  GtkWidget *PrintButton;
  GtkWidget *OKButton;
  GtkWidget *StartButton;
  GtkWidget *StopButton;

  GtkWidget *Brightness;
  GtkWidget *Gamma;
  GtkWidget *Speed;

  GdkColor white = {0, 0xffff, 0xffff, 0xffff};
  GtkStyle* ImageAreaStyle;

  /****************** ImageWindow****************************************/
  ImageWindow = gtk_dialog_new ();
  gtk_container_set_border_width (GTK_CONTAINER (ImageWindow), 5);
  gtk_window_set_title (GTK_WINDOW (ImageWindow), 
			"ConoMap - Interference Figure");

  ImageHBox = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(ImageWindow)->vbox), ImageHBox, 
		     FALSE, FALSE, 0);

  ActionBox = GTK_DIALOG (ImageWindow)->action_area; 
  gtk_container_set_border_width (GTK_CONTAINER (ActionBox), 5);

  /*********** ImageBox **************************************/
  ImageVBox = gtk_vbox_new(FALSE, 0);
  gtk_box_pack_start (GTK_BOX (ImageHBox), ImageVBox, FALSE, FALSE, 0);


  ImageFrame = gtk_frame_new(NULL);
  gtk_box_pack_start (GTK_BOX (ImageVBox), ImageFrame, FALSE, FALSE, 0);
  gtk_frame_set_shadow_type(GTK_FRAME (ImageFrame), GTK_SHADOW_IN);
  gtk_container_set_border_width (GTK_CONTAINER (ImageFrame), 10);

    

  ImageArea = gtk_drawing_area_new ();
  gtk_drawing_area_size(GTK_DRAWING_AREA(ImageArea), 
			total_image_size + 2*ImageMargin,
			total_image_size + 2*ImageMargin);

  gdk_color_alloc( gdk_colormap_get_system(), &white);
  ImageAreaStyle = gtk_style_copy(gtk_widget_get_default_style());
  ImageAreaStyle->bg[GTK_STATE_NORMAL] = white;
  gtk_widget_set_style(ImageArea, ImageAreaStyle); 


  gtk_container_add (GTK_CONTAINER (ImageFrame), ImageArea );

  gtk_object_set_data( GTK_OBJECT(ImageWindow), "ImageArea", ImageArea );
 				   
  ImageControlBox = gtk_vbox_new(FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (ImageControlBox), 10);
  gtk_box_pack_start (GTK_BOX (ImageHBox), ImageControlBox, FALSE, FALSE, 0);

  /*********** ImageControlBox **************************************/

  Brightness = pack_scale(ImageControlBox, _("Bright"), 60, 100, 
			      bright0, 0, bright0*2, 0.1, 0.1);
  Gamma = pack_scale(ImageControlBox, _("Gamma"), 60, 100,
				     gamma0, 1.0, 10.1, 0.1, 0.1);
  if(drawing_method.drawing_kind == Animation)
  {
     Speed = pack_scale(ImageControlBox, _("Speed"), 60, 100,
				 speed0, 10, 91, 1, 1);
     gtk_scale_set_digits(GTK_SCALE(Speed), 0);

     AnimationSpeed = speed0;
     AnimationAngle = 0;
     AnimationInterval = 1000*parameter.sampling_step/speed0;
     FrameRate = AnimationSpeed/parameter.sampling_step;

     AngleScale = pack_scale(ImageControlBox, _("Azimuth"), 60, 100, 
			   phi0, 0, 360, 1, 1);
     gtk_scale_set_digits(GTK_SCALE(AngleScale), 0);

     ImageControlButtonBox = gtk_hbox_new(TRUE, 0);
     gtk_box_pack_start (GTK_BOX (ImageVBox), ImageControlButtonBox, 
			 FALSE, FALSE, 0);     

     StartButton = pack_button(ImageControlButtonBox, _("Start"));
     StopButton = pack_button(ImageControlButtonBox, _("Stop"));
     AnimatingNow = FALSE;
  }

  /*************** ActionBox Contents ********************************/
  SaveButton = pack_button(ActionBox, _("Save"));
  PrintButton = pack_button(ActionBox,_( "Print"));
  OKButton = pack_button(ActionBox, _("Close"));

  /**************** signal connections ******************************/
  gtk_signal_connect (GTK_OBJECT (ImageArea), "expose_event",
                      GTK_SIGNAL_FUNC (expose_event), NULL);
  gtk_signal_connect (GTK_OBJECT (ImageWindow), "destroy",
                      GTK_SIGNAL_FUNC (ImageWindow_destroy), NULL);
  gtk_signal_connect (GTK_OBJECT (Brightness), "draw",
                      GTK_SIGNAL_FUNC (Brightness_draw), NULL);
  gtk_signal_connect (GTK_OBJECT (Gamma), "draw",
                      GTK_SIGNAL_FUNC (Gamma_draw), NULL);
  gtk_signal_connect (GTK_OBJECT (SaveButton), "clicked",
                      GTK_SIGNAL_FUNC (SaveButton_clicked), NULL);
  gtk_signal_connect (GTK_OBJECT (PrintButton), "clicked",
                      GTK_SIGNAL_FUNC (PrintButton_clicked), NULL);
  gtk_signal_connect (GTK_OBJECT (OKButton), "clicked",
                      GTK_SIGNAL_FUNC (OKButton_clicked), NULL);
  if(drawing_method.drawing_kind == Animation)
  {

     gtk_signal_connect (GTK_OBJECT (AngleScale), "expose_event",
			 GTK_SIGNAL_FUNC (expose_angle_scale), NULL);

     gtk_signal_connect (GTK_OBJECT (AngleScale), "draw",
			 GTK_SIGNAL_FUNC (Angle_draw), NULL);
     gtk_signal_connect (GTK_OBJECT (Speed), "draw",
			 GTK_SIGNAL_FUNC (Speed_draw),  NULL);
     gtk_signal_connect (GTK_OBJECT (StartButton), "clicked",
			 GTK_SIGNAL_FUNC (StartButton_clicked), NULL);
     gtk_signal_connect (GTK_OBJECT (StopButton), "clicked",
			 GTK_SIGNAL_FUNC (StopButton_clicked),  NULL);
     AnimationTimer = g_timer_new();
  }   

  gtk_widget_show_all(ImageWindow);

  return ImageWindow;
}

/*===========================================================================*
 * (Internal Proceures)                                                      *
 *           draw_image                                                      *
 *           animating                                                       *
 *===========================================================================*/
static void
draw_image()
{
   GdkRectangle ImageRect;
   
   ImageRect.x = ImageRect.y = 0;
   ImageRect.width = ImageArea->allocation.width;
   ImageRect.height = ImageArea->allocation.height;
   
   gtk_widget_draw(ImageArea, &ImageRect); 
}

static gboolean 
animating(gpointer data)
{
   GtkAdjustment *adjust;

   AnimationAngle += AnimationSpeed * g_timer_elapsed(AnimationTimer, NULL);
   g_timer_reset(AnimationTimer);
   
   if(AnimationAngle >= 360)
   {
      AnimationAngle = 0;
   }

   set_animation_image_index(AnimationAngle);

   adjust  = gtk_range_get_adjustment(GTK_RANGE(AngleScale)); 
   adjust->value = AnimationAngle;
   gtk_range_set_adjustment(GTK_RANGE(AngleScale), adjust);
   
   gtk_range_slider_update(GTK_RANGE(AngleScale));

   draw_image();
   return TRUE;
}

/*===========================================================================*
 * (CALLBACKS for  createImageWindow)                                        *
 *                                                                           *
 *        expose_event                                                       *
 *        ImageWindow_destroy                                                *
 *        Brightness_draw                                                    *
 *        Gamma_draw                                                         *
 *        OKButton_clicked                                                   *
 *        expose_angle_scale                                                 *
 *        Angle_draw                                                         *
 *        Speed_draw                                                         *
 *        StartButton_clicked                                                *
 *        StopButton_clicked                                                 *
 *        SaveButton_clicked                                                 *
 *        PrintButton_clicked                                                *
 *===========================================================================*/
gint
expose_event(GtkWidget *widget, GdkEventExpose *event)
{
   gdk_draw_pixmap(widget->window,
		   widget->style->fg_gc[GTK_WIDGET_STATE(widget)], pixmap,
		   event->area.x-ImageMargin, event->area.y-ImageMargin, 
		   event->area.x, event->area.y, 
		   event->area.width, event->area.height);
   return FALSE;
}

		   
void
ImageWindow_destroy (GtkObject *object, gpointer data)
{
   if(drawing_method.drawing_kind == Animation && AnimatingNow == TRUE)
   {
      gtk_timeout_remove(AnimationTimeout);
      g_timer_destroy(AnimationTimer);
   }
  
   image_memory_free();
   gtk_widget_destroy (GTK_WIDGET(object));
}

void
Brightness_draw (GtkWidget *widget, gpointer data)
{
   GtkAdjustment *adjust = gtk_range_get_adjustment(GTK_RANGE(widget));
   gfloat bright = adjust->value;
   change_brightness(bright);
   
   draw_image();
}

void 
Gamma_draw (GtkWidget *widget, gpointer data)
{
   GtkAdjustment *adjust = gtk_range_get_adjustment(GTK_RANGE(widget));
   gfloat gamma = adjust->value;
   change_gamma(gamma);
   
   draw_image();
}

void
OKButton_clicked (GtkButton *button, gpointer data)
{
   if(drawing_method.drawing_kind == Animation && AnimatingNow == TRUE)
     gtk_timeout_remove(AnimationTimeout);
   gtk_widget_destroy (gtk_widget_get_toplevel(GTK_WIDGET(button)));
}


void
expose_angle_scale (GtkWidget *widget, GdkEventExpose *event)
{
   gtk_range_clear_background (GTK_RANGE(widget));
}
 
void
Angle_draw (GtkWidget *widget, gpointer data)
{
   GtkAdjustment *adjust;

   if(AnimatingNow == TRUE)
     gtk_timeout_remove(AnimationTimeout);


   adjust = gtk_range_get_adjustment(GTK_RANGE(widget));
   set_animation_image_index(adjust->value); 
   AnimationAngle = adjust->value;
   g_timer_reset(AnimationTimer);

   if(AnimatingNow == TRUE)
     AnimationTimeout = gtk_timeout_add(AnimationInterval, 
                                        (GtkFunction)animating, NULL);
   draw_image();
}

void 
Speed_draw (GtkWidget *widget, gpointer data)
{
   GtkAdjustment *adjust;

   if(AnimatingNow == TRUE)
     gtk_timeout_remove(AnimationTimeout);

   adjust = gtk_range_get_adjustment(GTK_RANGE(widget));
   AnimationSpeed =adjust->value;
   AnimationInterval = 1000.0*parameter.sampling_step/adjust->value;
   FrameRate = AnimationSpeed/parameter.sampling_step;

   if(AnimatingNow == TRUE)
      AnimationTimeout = gtk_timeout_add(AnimationInterval, 
                                        (GtkFunction)animating, NULL);
}


void
StartButton_clicked (GtkButton *button, gpointer data)
{
   if(AnimatingNow==FALSE)
   {
      AnimationTimeout = gtk_timeout_add(AnimationInterval, 
					 (GtkFunction)animating, NULL);
      g_timer_start(AnimationTimer);
      AnimatingNow = TRUE;
   }
}

void
StopButton_clicked (GtkButton *button, gpointer data)
{
   if(AnimatingNow == TRUE)
   {
      gtk_timeout_remove(AnimationTimeout);
      AnimatingNow = FALSE;
   }
}

void
SaveButton_clicked (GtkButton *button, gpointer data)
{
  GtkWidget* fdialog;
  
  if(AnimatingNow == TRUE)
    return;

  fdialog = gtk_file_selection_new("Save as jpg or ppm file");

  gtk_signal_connect (GTK_OBJECT (fdialog), "destroy",
		      GTK_SIGNAL_FUNC (gtk_main_quit), NULL);

  gtk_signal_connect_object(
		GTK_OBJECT(GTK_FILE_SELECTION(fdialog)->cancel_button),
	        "clicked", (GtkSignalFunc)gtk_widget_destroy,
	         GTK_OBJECT(fdialog) );
  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(fdialog)->ok_button), 
                   "clicked", (GtkSignalFunc)fdialog_save, (gpointer)fdialog);

  if(drawing_method.drawing_kind == Animation)
    gtk_file_selection_set_filename(GTK_FILE_SELECTION(fdialog),"conomap.mpg");
  else
    gtk_file_selection_set_filename(GTK_FILE_SELECTION(fdialog),"conomap.jpg");

  
  gtk_widget_show(fdialog);
  gtk_grab_add(fdialog);
  gtk_main();
} 

void
PrintButton_clicked (GtkButton *button, gpointer data)
{
    static gchar* file_name = "_cono_map_print.pnm";
    gchar *print_command, print[100]; 

    if(AnimatingNow == TRUE)
      return;

    save_image(file_name, "ppm");

    create_text_entry_dialog( "Print", _("Print Command "), "lpr", _("Print"), 
			     &print_command );
    if(print_command != NULL)
    {
       sprintf( print, "%s %s", print_command, file_name); 
       system( print );

       remove(file_name);
   
       free(print_command);
   }
}

/*==========================================================*
 * CallBack and related Functions  for SaveButton_clicked() *
 *                                                          *
 *      fdialog_save                                        *
 *      mpeg_size_input                                     *
 *==========================================================*/
void 
fdialog_save(GtkWidget *widget, gpointer fdialog)
{
  gchar str[100];
  gchar file[100], *ext;
  guchar *f = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fdialog));
  
  strcpy(file, f);
  ext = get_file_extention(file);  
  
  if(ext == NULL)
  {
     gtk_widget_destroy(GTK_WIDGET(fdialog));
     draw_image();
     save_image(file, "ppm");
     return;
  }
  else if(strcasecmp(ext, "ppm") == 0 || strcasecmp(ext, "pnm") == 0 )
  {
     gtk_widget_destroy(GTK_WIDGET(fdialog));
     draw_image();
     save_image(file, "ppm");
     return;
  }
  else if(strcasecmp(ext, "jpg") == 0 || strcasecmp(ext, "jpeg") == 0)
  {
     gtk_widget_destroy(GTK_WIDGET(fdialog));
     draw_image();
     save_image(file, "jpg");
     return;
  }
  else if(strcasecmp(ext, "mpg") == 0 || strcasecmp(ext, "mpeg") == 0 )
  { 
     if (drawing_method.drawing_kind == Animation)
     {
	gtk_widget_destroy(GTK_WIDGET(fdialog));
	if(mpeg_size_input() == TRUE)
	{
	   save_image(file, "mpg");
	   return;
	}
     }
     else
     {
	sprintf(str, _("%s files are supported only in animation mode."), ext);
	MessageDialog(str);
     }
  }
  else
  {
     sprintf( str, _("%s files are not supported."), ext);
     MessageDialog(str);
  }
}
   

/*=====================================*
 * STATIC VARIABLE for mpeg_size_input *
 *=====================================*/
static gboolean mpeg_stat;

/*===============================*
 * CALLBACKS for mpeg_size_input *
 *===============================*/
static gboolean mpeg_delete(GtkObject *object, gpointer data);
static void mpeg_cancel(GtkButton *button, gpointer data);
static void mpeg_size(GtkButton* button, gpointer data);

gboolean 
mpeg_size_input()
{
   GtkWidget *dialog;
   GtkWidget* box, *action;
   GtkWidget* spin;
   GtkWidget* save, *cancel;
   int size, min_size, max_size;

   size = 240;

   min_size = image_size/16;
   if(image_size%16) min_size++;
   min_size *= 16;

   max_size = 480;

   dialog = gtk_dialog_new();
   gtk_window_set_title (GTK_WINDOW (dialog), "MPEG file size");
   
   box = GTK_DIALOG (dialog)->vbox;
   action = GTK_DIALOG (dialog)->action_area;
   spin = pack_spin(box, _("Image Height(in pixel): "), 185, 60,
		    240, min_size, max_size, 16, 16, 0);

   save = pack_button(action, _("OK"));
   gtk_object_set_data( GTK_OBJECT(save), "spin", spin);
   cancel = pack_button(action, _("Cancel"));

   gtk_signal_connect(GTK_OBJECT(dialog), "delete_event", 
		      GTK_SIGNAL_FUNC(mpeg_delete), NULL);
   gtk_signal_connect(GTK_OBJECT(cancel), "clicked",
		      GTK_SIGNAL_FUNC(mpeg_cancel), NULL);
   gtk_signal_connect(GTK_OBJECT(save), "clicked", 
		      GTK_SIGNAL_FUNC(mpeg_size), NULL);

   gtk_widget_show_all(dialog);
   gtk_grab_add(GTK_WIDGET(dialog));
   gtk_main();
   gtk_widget_destroy(dialog);
   return mpeg_stat;
}   
   
static gboolean      
mpeg_delete(GtkObject *object, gpointer data)
{
   mpeg_stat = FALSE;
   return FALSE;
}

static void    
mpeg_cancel(GtkButton *button, gpointer data)
{
   mpeg_stat = FALSE;
   gtk_main_quit();
}

static void
mpeg_size(GtkButton* button, gpointer data)
{
   GtkWidget* spin = gtk_object_get_data(GTK_OBJECT(button), "spin");

   mpeg_image_size = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin));
   mpeg_stat = TRUE;
   gtk_main_quit();
}

/*==========================================================================*
  (FUNCTION called from PrintButton_clicked )                               *
            create_text_entry_dialog                                        *
 *==========================================================================*/

/*========================================*
 * CALLBACKS for create_text_entry_dialog *
 *========================================*/
static gboolean entry_delete(GtkObject *object, gpointer data);
static void entry_cancel(GtkButton *button, gpointer data);
static void get_entry_str(GtkWidget *widget, gchar **str);


static void 
create_text_entry_dialog( gchar* title, gchar* label_str, 
	gchar* default_entry_str, gchar* action_str, gchar** get_str )
{
   GtkWidget *dialog;
   GtkWidget *entry;
   GtkWidget* hbox, *action;
   GtkWidget* label;
   GtkWidget* ok, *cancel;

   dialog = gtk_dialog_new();
   gtk_window_set_title (GTK_WINDOW (dialog), title);
   
   action = GTK_DIALOG (dialog)->action_area;

   hbox = gtk_hbox_new(FALSE, 0);
   gtk_container_set_border_width( GTK_CONTAINER(hbox), 15);
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox, FALSE, FALSE, 5);

   label = gtk_label_new(label_str);
   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
   
   entry = gtk_entry_new();
   gtk_widget_set_usize(GTK_WIDGET(entry), 200, 20);
   gtk_entry_set_max_length (GTK_ENTRY(entry), 50);   
   gtk_entry_prepend_text(GTK_ENTRY(entry), default_entry_str);
   gtk_box_pack_start(GTK_BOX(hbox), entry, FALSE, FALSE, 0);
   gtk_object_set_data( GTK_OBJECT(dialog), "Entry", entry);

   ok = pack_button(action, action_str);
   cancel = pack_button(action, _("Cancel"));

   gtk_signal_connect(GTK_OBJECT(dialog), "delete_event", 
		      GTK_SIGNAL_FUNC(entry_delete), NULL);
   gtk_signal_connect(GTK_OBJECT(cancel), "clicked",
		      GTK_SIGNAL_FUNC(entry_cancel), NULL);
   gtk_signal_connect(GTK_OBJECT(ok), "clicked", 
		      GTK_SIGNAL_FUNC(get_entry_str), get_str);

   gtk_widget_show_all(dialog);
   gtk_grab_add(GTK_WIDGET(dialog));
   gtk_main();
}
static gboolean      
entry_delete(GtkObject *object, gpointer data)
{
   return FALSE;
}

static void    
entry_cancel(GtkButton *button, gpointer data)
{
   gtk_widget_destroy (gtk_widget_get_toplevel(GTK_WIDGET(button)));
}

static void
get_entry_str(GtkWidget *widget, gchar **str)
{
   GtkWidget* entry, *dialog;
   dialog = gtk_widget_get_toplevel(widget);
   entry = gtk_object_get_data(GTK_OBJECT(dialog), "Entry");
   if(entry)
   {
      *str = strdup(gtk_entry_get_text(GTK_ENTRY(entry)));
   }
   gtk_main_quit();
   gtk_widget_destroy (gtk_widget_get_toplevel(widget));
}


